# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v3 import roles_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestRolesClient(base.BaseServiceTest):

    FAKE_ROLE_ID = "1"
    FAKE_ROLE_NAME = "test"
    FAKE_DOMAIN_ID = "1"

    FAKE_ROLE_ID_2 = "2"
    FAKE_ROLE_NAME_2 = "test2"

    FAKE_ROLE_ID_3 = "3"
    FAKE_ROLE_NAME_3 = "test3"

    FAKE_ROLE_ID_4 = "4"
    FAKE_ROLE_NAME_4 = "test4"

    FAKE_ROLE_ID_5 = "5"
    FAKE_ROLE_NAME_5 = "test5"

    FAKE_ROLE_ID_6 = "6"
    FAKE_ROLE_NAME_6 = "test6"

    FAKE_ROLE_INFO = {
        "role": {
            "domain_id": FAKE_DOMAIN_ID,
            "id": FAKE_ROLE_ID,
            "name": FAKE_ROLE_NAME,
            "links": {
                "self": "http://example.com/identity/v3/roles/%s" % (
                    FAKE_ROLE_ID)
            }
        }
    }

    FAKE_ROLE_INFO_2 = {
        "role": {
            "domain_id": FAKE_DOMAIN_ID,
            "id": FAKE_ROLE_ID_2,
            "name": FAKE_ROLE_NAME_2,
            "links": {
                "self": "http://example.com/identity/v3/roles/%s" % (
                    FAKE_ROLE_ID_2)
            }
        }
    }

    FAKE_LIST_ROLES = {"roles": [FAKE_ROLE_INFO, FAKE_ROLE_INFO_2]}

    FAKE_ROLE_INFERENCE_RULE = {
        "role_inference": {
            "prior_role": {
                "id": FAKE_ROLE_ID,
                "name": FAKE_ROLE_NAME,
                "links": {
                    "self": "http://example.com/identity/v3/roles/%s" % (
                        FAKE_ROLE_ID)
                }
            },
            "implies": {
                "id": FAKE_ROLE_ID_2,
                "name": FAKE_ROLE_NAME_2,
                "links": {
                    "self": "http://example.com/identity/v3/roles/%s" % (
                        FAKE_ROLE_ID_2)
                }
            }
        },
        "links": {
            "self": "http://example.com/identity/v3/roles/"
                    "%s/implies/%s" % (FAKE_ROLE_ID, FAKE_ROLE_ID_2)
        }
    }

    COMMON_FAKE_LIST_ROLE_INFERENCE_RULES = [
        {
            "prior_role": {
                "id": FAKE_ROLE_ID,
                "name": FAKE_ROLE_NAME,
                "links": {
                    "self": "http://example.com/identity/v3/roles/%s" % (
                        FAKE_ROLE_ID)
                }
            },
            "implies": [
                {
                    "id": FAKE_ROLE_ID_2,
                    "name": FAKE_ROLE_NAME_2,
                    "links": {
                        "self": "http://example.com/identity/v3/roles/%s" % (
                            FAKE_ROLE_ID_2)
                    }
                },
                {
                    "id": FAKE_ROLE_ID_3,
                    "name": FAKE_ROLE_NAME_3,
                    "links": {
                        "self": "http://example.com/identity/v3/roles/%s" % (
                            FAKE_ROLE_ID_3)
                    }
                }
            ]
        },
        {
            "prior_role": {
                "id": FAKE_ROLE_ID_4,
                "name": FAKE_ROLE_NAME_4,
                "links": {
                    "self": "http://example.com/identity/v3/roles/%s" % (
                        FAKE_ROLE_ID_4)
                }
            },
            "implies": [
                {
                    "id": FAKE_ROLE_ID_5,
                    "name": FAKE_ROLE_NAME_5,
                    "links": {
                        "self": "http://example.com/identity/v3/roles/%s" % (
                            FAKE_ROLE_ID_5)
                    }
                },
                {
                    "id": FAKE_ROLE_ID_6,
                    "name": FAKE_ROLE_NAME_6,
                    "links": {
                        "self": "http://example.com/identity/v3/roles/%s" % (
                            FAKE_ROLE_ID_6)
                    }
                }
            ]
        }
    ]

    FAKE_LIST_ROLE_INFERENCE_RULES = {
        "role_inference": COMMON_FAKE_LIST_ROLE_INFERENCE_RULES[0],
        "links": {
            "self": "http://example.com/identity/v3/roles/"
                    "%s/implies" % FAKE_ROLE_ID
        }
    }

    FAKE_LIST_ALL_ROLE_INFERENCE_RULES = {
        "role_inferences": COMMON_FAKE_LIST_ROLE_INFERENCE_RULES,
        "links": {
            "self": "http://example.com/identity/v3/role_inferences"
        }
    }

    def setUp(self):
        super(TestRolesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = roles_client.RolesClient(fake_auth,
                                               'identity', 'regionOne')

    def _test_create_role(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_role,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_ROLE_INFO,
            bytes_body,
            domain_id=self.FAKE_DOMAIN_ID,
            name=self.FAKE_ROLE_NAME,
            status=201)

    def _test_show_role(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_role,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ROLE_INFO,
            bytes_body,
            role_id=self.FAKE_ROLE_ID)

    def _test_list_roles(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_roles,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body)

    def _test_update_role(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_role,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_ROLE_INFO,
            bytes_body,
            role_id=self.FAKE_ROLE_ID,
            name=self.FAKE_ROLE_NAME)

    def _test_create_user_role_on_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_user_role_on_project,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def _test_create_user_role_on_domain(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_user_role_on_domain,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def _test_create_user_role_on_system(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_user_role_on_system,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            user_id="123",
            role_id="1234",
            status=204)

    def _test_list_user_roles_on_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_user_roles_on_project,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123")

    def _test_list_user_roles_on_domain(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_user_roles_on_domain,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123")

    def _test_list_user_roles_on_system(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_user_roles_on_system,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            user_id="123")

    def _test_create_group_role_on_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group_role_on_project,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def _test_create_group_role_on_domain(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group_role_on_domain,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def _test_create_group_role_on_system(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group_role_on_system,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            group_id="123",
            role_id="1234",
            status=204)

    def _test_list_group_roles_on_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_group_roles_on_project,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123")

    def _test_list_group_roles_on_domain(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_group_roles_on_domain,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123")

    def _test_list_group_roles_on_system(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_group_roles_on_system,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123")

    def _test_create_role_inference_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_role_inference_rule,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_ROLE_INFERENCE_RULE,
            bytes_body,
            status=201,
            prior_role=self.FAKE_ROLE_ID,
            implies_role=self.FAKE_ROLE_ID_2)

    def _test_show_role_inference_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_role_inference_rule,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ROLE_INFERENCE_RULE,
            bytes_body,
            prior_role=self.FAKE_ROLE_ID,
            implies_role=self.FAKE_ROLE_ID_2)

    def _test_list_role_inferences_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_role_inferences_rules,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLE_INFERENCE_RULES,
            bytes_body,
            prior_role=self.FAKE_ROLE_ID)

    def _test_list_all_role_inference_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_all_role_inference_rules,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ALL_ROLE_INFERENCE_RULES,
            bytes_body)

    def test_create_role_with_str_body(self):
        self._test_create_role()

    def test_create_role_with_bytes_body(self):
        self._test_create_role(bytes_body=True)

    def test_show_role_with_str_body(self):
        self._test_show_role()

    def test_show_role_with_bytes_body(self):
        self._test_show_role(bytes_body=True)

    def test_list_roles_with_str_body(self):
        self._test_list_roles()

    def test_list_roles_with_bytes_body(self):
        self._test_list_roles(bytes_body=True)

    def test_update_role_with_str_body(self):
        self._test_update_role()

    def test_update_role_with_bytes_body(self):
        self._test_update_role(bytes_body=True)

    def test_delete_role(self):
        self.check_service_client_function(
            self.client.delete_role,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            role_id=self.FAKE_ROLE_ID,
            status=204)

    def test_create_user_role_on_project_with_str_body(self):
        self._test_create_user_role_on_project()

    def test_create_user_role_on_project_with_bytes_body(self):
        self._test_create_user_role_on_project(bytes_body=True)

    def test_create_user_role_on_domain_with_str_body(self):
        self._test_create_user_role_on_domain()

    def test_create_user_role_on_domain_with_bytes_body(self):
        self._test_create_user_role_on_domain(bytes_body=True)

    def test_create_group_role_on_domain_with_str_body(self):
        self._test_create_group_role_on_domain()

    def test_create_group_role_on_domain_with_bytes_body(self):
        self._test_create_group_role_on_domain(bytes_body=True)

    def test_list_user_roles_on_project_with_str_body(self):
        self._test_list_user_roles_on_project()

    def test_list_user_roles_on_project_with_bytes_body(self):
        self._test_list_user_roles_on_project(bytes_body=True)

    def test_list_user_roles_on_domain_with_str_body(self):
        self._test_list_user_roles_on_domain()

    def test_list_user_roles_on_domain_with_bytes_body(self):
        self._test_list_user_roles_on_domain(bytes_body=True)

    def test_list_group_roles_on_domain_with_str_body(self):
        self._test_list_group_roles_on_domain()

    def test_list_group_roles_on_domain_with_bytes_body(self):
        self._test_list_group_roles_on_domain(bytes_body=True)

    def test_delete_role_from_user_on_project(self):
        self.check_service_client_function(
            self.client.delete_role_from_user_on_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_delete_role_from_user_on_domain(self):
        self.check_service_client_function(
            self.client.delete_role_from_user_on_domain,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_delete_role_from_user_on_system(self):
        self.check_service_client_function(
            self.client.delete_role_from_user_on_system,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            user_id="123",
            role_id="1234",
            status=204)

    def test_delete_role_from_group_on_project(self):
        self.check_service_client_function(
            self.client.delete_role_from_group_on_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_delete_role_from_group_on_domain(self):
        self.check_service_client_function(
            self.client.delete_role_from_group_on_domain,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_delete_role_from_group_on_system(self):
        self.check_service_client_function(
            self.client.delete_role_from_group_on_system,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            group_id="123",
            role_id="1234",
            status=204)

    def test_check_user_role_existence_on_project(self):
        self.check_service_client_function(
            self.client.check_user_role_existence_on_project,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_check_user_role_existence_on_domain(self):
        self.check_service_client_function(
            self.client.check_user_role_existence_on_domain,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_check_user_role_existence_on_system(self):
        self.check_service_client_function(
            self.client.check_user_role_existence_on_system,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            user_id="123",
            role_id="1234",
            status=204)

    def test_check_role_from_group_on_project_existence(self):
        self.check_service_client_function(
            self.client.check_role_from_group_on_project_existence,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_check_role_from_group_on_domain_existence(self):
        self.check_service_client_function(
            self.client.check_role_from_group_on_domain_existence,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_check_role_from_group_on_system_existence(self):
        self.check_service_client_function(
            self.client.check_role_from_group_on_system_existence,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            group_id="123",
            role_id="1234",
            status=204)

    def test_create_role_inference_rule_with_str_body(self):
        self._test_create_role_inference_rule()

    def test_create_role_inference_rule_with_bytes_body(self):
        self._test_create_role_inference_rule(bytes_body=True)

    def test_show_role_inference_rule_with_str_body(self):
        self._test_show_role_inference_rule()

    def test_show_role_inference_rule_with_bytes_body(self):
        self._test_show_role_inference_rule(bytes_body=True)

    def test_list_role_inferences_rules_with_str_body(self):
        self._test_list_role_inferences_rules()

    def test_list_role_inferences_rules_with_bytes_body(self):
        self._test_list_role_inferences_rules(bytes_body=True)

    def test_check_role_inference_rule(self):
        self.check_service_client_function(
            self.client.check_role_inference_rule,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            status=204,
            prior_role=self.FAKE_ROLE_ID,
            implies_role=self.FAKE_ROLE_ID_2)

    def test_delete_role_inference_rule(self):
        self.check_service_client_function(
            self.client.delete_role_inference_rule,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            prior_role=self.FAKE_ROLE_ID,
            implies_role=self.FAKE_ROLE_ID_2)

    def test_list_all_role_inference_rules_with_str_body(self):
        self._test_list_all_role_inference_rules()

    def test_list_all_role_inference_rules_with_bytes_body(self):
        self._test_list_all_role_inference_rules(bytes_body=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class VolumesSnapshotNegativeTestJSON(base.BaseVolumeTest):
    """Negative tests of volume snapshot"""

    @classmethod
    def skip_checks(cls):
        super(VolumesSnapshotNegativeTestJSON, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e3e466af-70ab-4f4b-a967-ab04e3532ea7')
    def test_create_snapshot_with_nonexistent_volume_id(self):
        """Test creating snapshot from non existent volume should fail"""
        s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-snap')
        self.assertRaises(lib_exc.NotFound,
                          self.snapshots_client.create_snapshot,
                          volume_id=data_utils.rand_uuid(),
                          display_name=s_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('bb9da53e-d335-4309-9c15-7e76fd5e4d6d')
    def test_create_snapshot_without_passing_volume_id(self):
        """Test creating snapshot without passing volume_id should fail"""
        # Create a snapshot without passing volume id
        s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-snap')
        self.assertRaises(lib_exc.NotFound,
                          self.snapshots_client.create_snapshot,
                          volume_id=None, display_name=s_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('677863d1-34f9-456d-b6ac-9924f667a7f4')
    def test_volume_from_snapshot_decreasing_size(self):
        """Test creating volume from snapshot with decreasing size

        creating volume from snapshot with decreasing size should fail.
        """
        # Creates a volume a snapshot passing a size different from the source
        src_size = CONF.volume.volume_size * 2

        src_vol = self.create_volume(size=src_size)
        src_snap = self.create_snapshot(src_vol['id'])

        # Destination volume smaller than source
        self.assertRaises(lib_exc.BadRequest,
                          self.volumes_client.create_volume,
                          size=CONF.volume.volume_size,
                          snapshot_id=src_snap['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8fd92339-e22f-4591-86b4-1e2215372a40')
    def test_list_snapshot_invalid_param_limit(self):
        """Test listing snapshots with invalid limit param should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.snapshots_client.list_snapshots,
                          limit='invalid')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('27b5f37f-bf69-4e8c-986e-c44f3d6819b8')
    def test_list_snapshots_invalid_param_sort(self):
        """Test listing snapshots with invalid sort key should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.snapshots_client.list_snapshots,
                          sort_key='invalid')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b68deeda-ca79-4a32-81af-5c51179e553a')
    def test_list_snapshots_invalid_param_marker(self):
        """Test listing snapshots with invalid marker should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.snapshots_client.list_snapshots,
                          marker=data_utils.rand_uuid())



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import fixtures
import testtools


class BaseTestCase(testtools.testcase.WithAttributes, testtools.TestCase):
    setUpClassCalled = False

    # NOTE(sdague): log_format is defined inline here instead of using the oslo
    # default because going through the config path recouples config to the
    # stress tests too early, and depending on testr order will fail unit tests
    log_format = ('%(asctime)s %(process)d %(levelname)-8s '
                  '[%(name)s] %(message)s')

    @classmethod
    def setUpClass(cls):
        if hasattr(super(BaseTestCase, cls), 'setUpClass'):
            super(BaseTestCase, cls).setUpClass()
        cls.setUpClassCalled = True

    @classmethod
    def tearDownClass(cls):
        if hasattr(super(BaseTestCase, cls), 'tearDownClass'):
            super(BaseTestCase, cls).tearDownClass()

    def setUp(self):
        super(BaseTestCase, self).setUp()
        if not self.setUpClassCalled:
            raise RuntimeError("setUpClass does not calls the super's "
                               "setUpClass in {!r}".format(type(self)))
        test_timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
                os.environ.get('OS_STDERR_CAPTURE') == '1'):
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))
        if (os.environ.get('OS_LOG_CAPTURE') != 'False' and
                os.environ.get('OS_LOG_CAPTURE') != '0'):
            self.useFixture(fixtures.LoggerFixture(nuke_handlers=False,
                                                   format=self.log_format,
                                                   level=None))



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import namespaces_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestNamespacesClient(base.BaseServiceTest):
    FAKE_CREATE_NAMESPACE = {
        "created_at": "2016-05-19T16:05:48Z",
        "description": "A metadata definitions namespace.",
        "display_name": "An Example Namespace",
        "namespace": "FredCo::SomeCategory::Example",
        "owner": "c60b1d57c5034e0d86902aedf8c49be0",
        "protected": True,
        "schema": "/v2/schemas/metadefs/namespace",
        "self": "/v2/metadefs/namespaces/"
                "FredCo::SomeCategory::Example",
        "updated_at": "2016-05-19T16:05:48Z",
        "visibility": "public"
    }

    FAKE_SHOW_NAMESPACE = {
        "created_at": "2016-06-28T14:57:10Z",
        "description": "The libvirt compute driver options.",
        "display_name": "libvirt Driver Options",
        "namespace": "OS::Compute::Libvirt",
        "owner": "admin",
        "properties": {
            "boot_menu": {
                "description": "If true, enables the BIOS bootmenu.",
                "enum": [
                    "true",
                    "false"
                ],
                "title": "Boot Menu",
                "type": "string"
            },
            "serial_port_count": {
                "description": "Specifies the count of serial ports.",
                "minimum": 0,
                "title": "Serial Port Count",
                "type": "integer"
            }
        },
        "protected": True,
        "resource_type_associations": [
            {
                "created_at": "2016-06-28T14:57:10Z",
                "name": "OS::Glance::Image",
                "prefix": "hw_"
            },
            {
                "created_at": "2016-06-28T14:57:10Z",
                "name": "OS::Nova::Flavor",
                "prefix": "hw:"
            }
        ],
        "schema": "/v2/schemas/metadefs/namespace",
        "self": "/v2/metadefs/namespaces/OS::Compute::Libvirt",
        "visibility": "public"
    }

    FAKE_LIST_NAMESPACES = {
        "first": "/v2/metadefs/namespaces?sort_key=created_at&"
                 "sort_dir=asc",
        "namespaces": [
            {
                "created_at": "2014-08-28T17:13:06Z",
                "description": "OS::Compute::Libvirt",
                "display_name": "libvirt Driver Options",
                "namespace": "OS::Compute::Libvirt",
                "owner": "admin",
                "protected": True,
                "resource_type_associations": [
                    {
                        "created_at": "2014-08-28T17:13:06Z",
                        "name": "OS::Glance::Image",
                        "updated_at": "2014-08-28T17:13:06Z"
                    }
                ],
                "schema": "/v2/schemas/metadefs/namespace",
                "self": "/v2/metadefs/namespaces/OS::Compute::Libvirt",
                "updated_at": "2014-08-28T17:13:06Z",
                "visibility": "public"
            },
            {
                "created_at": "2014-08-28T17:13:06Z",
                "description": "OS::Compute::Quota",
                "display_name": "Flavor Quota",
                "namespace": "OS::Compute::Quota",
                "owner": "admin",
                "protected": True,
                "resource_type_associations": [
                    {
                        "created_at": "2014-08-28T17:13:06Z",
                        "name": "OS::Nova::Flavor",
                        "updated_at": "2014-08-28T17:13:06Z"
                    }
                ],
                "schema": "/v2/schemas/metadefs/namespace",
                "self": "/v2/metadefs/namespaces/OS::Compute::Quota",
                "updated_at": "2014-08-28T17:13:06Z",
                "visibility": "public"
            }
        ],
        "schema": "/v2/schemas/metadefs/namespaces"
    }

    FAKE_UPDATE_NAMESPACE = {
        "namespace": "OS::Compute::Hypervisor",
        "visibility": "public",
        "description": "Tempest",
        "display_name": u"\u2740(*\xb4\u25e2`*)\u2740",
        "protected": True
    }

    def setUp(self):
        super(TestNamespacesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = namespaces_client.NamespacesClient(fake_auth,
                                                         'image', 'regionOne')

    def _test_show_namespace(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_namespace,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_NAMESPACE,
            bytes_body,
            namespace="OS::Compute::Hypervisor")

    def _test_list_namespaces(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_namespaces,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_NAMESPACES,
            bytes_body)

    def _test_create_namespace(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_namespace,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_NAMESPACE,
            bytes_body,
            namespace="OS::Compute::Hypervisor",
            visibility="public", description="Tempest",
            display_name=u"\u2740(*\xb4\u25e1`*)\u2740", protected=True,
            status=201)

    def _test_update_namespace(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_namespace,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_NAMESPACE,
            bytes_body,
            namespace="OS::Compute::Hypervisor",
            display_name=u"\u2740(*\xb4\u25e2`*)\u2740", protected=True)

    def test_show_namespace_with_str_body(self):
        self._test_show_namespace()

    def test_show_namespace_with_bytes_body(self):
        self._test_show_namespace(bytes_body=True)

    def test_list_namespaces_with_str_body(self):
        self._test_list_namespaces()

    def test_list_namespaces_with_bytes_body(self):
        self._test_list_namespaces(bytes_body=True)

    def test_create_namespace_with_str_body(self):
        self._test_create_namespace()

    def test_create_namespace_with_bytes_body(self):
        self._test_create_namespace(bytes_body=True)

    def test_delete_namespace(self):
        self.check_service_client_function(
            self.client.delete_namespace,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, namespace="OS::Compute::Hypervisor", status=204)

    def test_update_namespace_with_str_body(self):
        self._test_update_namespace()

    def test_update_namespace_with_bytes_body(self):
        self._test_update_namespace(bytes_body=True)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

delete_message = {
    'status_code': [204],
}

common_show_message = {
    'type': 'object',
    'properties': {
        'request_id': {'type': 'string'},
        'message_level': {'type': 'string'},
        'links': parameter_types.links,
        'event_id': {'type': 'string'},
        'created_at': parameter_types.date_time,
        'guaranteed_until': parameter_types.date_time,
        'resource_uuid': {'type': 'string', 'format': 'uuid'},
        'id': {'type': 'string', 'format': 'uuid'},
        'resource_type': {'type': 'string'},
        'user_message': {'type': 'string'}},
    'additionalProperties': False,
    'required': ['request_id', 'message_level', 'event_id', 'created_at',
                 'id', 'user_message'],
}

show_message = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'message': common_show_message
        },
        'additionalProperties': False,
        'required': ['message']
    }
}

list_messages = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'messages': {
                'type': 'array',
                'items': common_show_message
            },
        },
        'additionalProperties': False,
        'required': ['messages']
    }
}



# Copyright 2013 Huawei Technologies Co.,LTD.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import tempest_fixtures as fixtures
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ServersAdminNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Negative Tests of Servers API using admin privileges"""

    @classmethod
    def setup_clients(cls):
        super(ServersAdminNegativeTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.servers_client
        cls.quotas_client = cls.os_admin.quotas_client

    @classmethod
    def resource_setup(cls):
        super(ServersAdminNegativeTestJSON, cls).resource_setup()
        cls.tenant_id = cls.client.tenant_id

        server = cls.create_test_server(wait_until='ACTIVE')
        cls.s1_id = server['id']

    @decorators.idempotent_id('28dcec23-f807-49da-822c-56a92ea3c687')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @decorators.attr(type=['negative'])
    def test_resize_server_using_overlimit_ram(self):
        """Test resizing server using over limit ram should fail"""
        # NOTE(mriedem): Avoid conflicts with os-quota-class-sets tests.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        quota_set = self.quotas_client.show_quota_set(
            self.tenant_id)['quota_set']
        ram = quota_set['ram']
        if ram == -1:
            raise self.skipException("ram quota set is -1,"
                                     " cannot test overlimit")
        ram += 1
        vcpus = 1
        disk = 5
        flavor_ref = self.create_flavor(ram=ram, vcpus=vcpus, disk=disk)
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.client.resize_server,
                          self.s1_id,
                          flavor_ref['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7fcadfab-bd6a-4753-8db7-4a51e51aade9')
    def test_restore_server_invalid_state(self):
        """Restore-deleting a server not in 'soft-delete' state should fail

        We can restore a soft deleted server, but can't restore a server that
        is not in 'soft-delete' state.
        """
        self.assertRaises(lib_exc.Conflict,
                          self.client.restore_soft_deleted_server,
                          self.s1_id)

    @decorators.idempotent_id('7368a427-2f26-4ad9-9ba9-911a0ec2b0db')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @decorators.attr(type=['negative'])
    def test_resize_server_using_overlimit_vcpus(self):
        """Test resizing server using over limit vcpus should fail"""
        # NOTE(mriedem): Avoid conflicts with os-quota-class-sets tests.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        quota_set = self.quotas_client.show_quota_set(
            self.tenant_id)['quota_set']
        vcpus = quota_set['cores']
        if vcpus == -1:
            raise self.skipException("cores quota set is -1,"
                                     " cannot test overlimit")
        vcpus += 1
        ram = 512
        disk = 5
        flavor_ref = self.create_flavor(ram=ram, vcpus=vcpus, disk=disk)
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.client.resize_server,
                          self.s1_id,
                          flavor_ref['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b0b4d8af-1256-41ef-9ee7-25f1c19dde80')
    def test_reset_state_server_invalid_state(self):
        """Test resetting server state to invalid state value should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.client.reset_state, self.s1_id,
                          state='invalid')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('4cdcc984-fab0-4577-9a9d-6d558527ee9d')
    def test_reset_state_server_invalid_type(self):
        """Test resetting server state to invalid state type should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.client.reset_state, self.s1_id,
                          state=1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e741298b-8df2-46f0-81cb-8f814ff2504c')
    def test_reset_state_server_nonexistent_server(self):
        """Test resetting a non existent server's state should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.reset_state, '999', state='error')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('46a4e1ca-87ae-4d28-987a-1b6b136a0221')
    def test_migrate_non_existent_server(self):
        """Test migrating a non existent server should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.migrate_server,
                          data_utils.rand_uuid())

    @decorators.idempotent_id('b0b17f83-d14e-4fc4-8f31-bcc9f3cfa629')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration not available.')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @decorators.attr(type=['negative'])
    def test_migrate_server_invalid_state(self):
        """Test migrating a server with invalid state should fail"""
        # create server.
        server = self.create_test_server(wait_until='ACTIVE')
        server_id = server['id']
        # suspend the server.
        self.client.suspend_server(server_id)
        waiters.wait_for_server_status(self.client,
                                       server_id, 'SUSPENDED')
        # migrate a suspended server should fail
        self.assertRaises(lib_exc.Conflict,
                          self.client.migrate_server,
                          server_id)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.compute import certificates_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestCertificatesClient(base.BaseServiceTest):

    FAKE_CERTIFICATE = {
        "certificate": {
            "data": "-----BEGIN----MIICyzCCAjSgAwI----END CERTIFICATE-----\n",
            "private_key": None
        }
    }

    def setUp(self):
        super(TestCertificatesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = certificates_client.CertificatesClient(
            fake_auth, 'compute', 'regionOne')

    def _test_show_certificate(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_certificate,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CERTIFICATE,
            bytes_body,
            certificate_id="fake-id")

    def test_show_certificate_with_str_body(self):
        self._test_show_certificate()

    def test_show_certificate_with_bytes_body(self):
        self._test_show_certificate(bytes_body=True)

    def _test_create_certificate(self, bytes_body=False):
        cert = copy.deepcopy(self.FAKE_CERTIFICATE)
        cert['certificate']['private_key'] = "my_private_key"
        self.check_service_client_function(
            self.client.create_certificate,
            'tempest.lib.common.rest_client.RestClient.post',
            cert,
            bytes_body)

    def test_create_certificate_with_str_body(self):
        self._test_create_certificate()

    def test_create_certificate_with_bytes_body(self):
        self._test_create_certificate(bytes_body=True)



# Copyright 2014 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib import decorators


class AvailabilityZoneTestJSON(base.BaseVolumeTest):
    """Tests Availability Zone API List"""

    @decorators.idempotent_id('01f1ae88-eba9-4c6b-a011-6f7ace06b725')
    def test_get_availability_zone_list(self):
        """Test listing volume available zones"""
        availability_zone = (
            self.availability_zone_client.list_availability_zones()
            ['availabilityZoneInfo'])
        self.assertNotEmpty(availability_zone)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class RoleAssignmentsClient(rest_client.RestClient):
    api_version = "v3"

    def list_role_assignments(self, effective=False, **kwargs):
        """List role assignments.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#list-role-assignments

        :param effective: If True, returns the effective assignments, including
                          any assignments gained by virtue of group membership
                          or inherited roles.
        """
        url = 'role_assignments'
        if kwargs:
            # NOTE(rodrigods): "effective" is a key-only query parameter and
            # is treated below.
            if 'effective' in kwargs:
                del kwargs['effective']
            url += '?%s' % urllib.urlencode(kwargs)
        if effective:
            url += '&effective'

        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from tempest.lib.services.compute import instance_usage_audit_log_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestInstanceUsagesAuditLogClient(base.BaseServiceTest):

    FAKE_AUDIT_LOG = {
        "hosts_not_run": [
            "f4eb7cfd155f4574967f8b55a7faed75"
        ],
        "log": {},
        "num_hosts": 1,
        "num_hosts_done": 0,
        "num_hosts_not_run": 1,
        "num_hosts_running": 0,
        "overall_status": "0 of 1 hosts done. 0 errors.",
        "period_beginning": "2012-12-01 00:00:00",
        "period_ending": "2013-01-01 00:00:00",
        "total_errors": 0,
        "total_instances": 0
    }

    def setUp(self):
        super(TestInstanceUsagesAuditLogClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = (instance_usage_audit_log_client.
                       InstanceUsagesAuditLogClient(fake_auth, 'compute',
                                                    'regionOne'))

    def _test_list_instance_usage_audit_logs(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_instance_usage_audit_logs,
            'tempest.lib.common.rest_client.RestClient.get',
            {"instance_usage_audit_logs": self.FAKE_AUDIT_LOG},
            bytes_body)

    def test_list_instance_usage_audit_logs_with_str_body(self):
        self._test_list_instance_usage_audit_logs()

    def test_list_instance_usage_audit_logs_with_bytes_body(self):
        self._test_list_instance_usage_audit_logs(bytes_body=True)

    def _test_show_instance_usage_audit_log(self, bytes_body=False):
        before_time = datetime.datetime(2012, 12, 1, 0, 0)
        self.check_service_client_function(
            self.client.show_instance_usage_audit_log,
            'tempest.lib.common.rest_client.RestClient.get',
            {"instance_usage_audit_log": self.FAKE_AUDIT_LOG},
            bytes_body,
            time_before=before_time)

    def test_show_instance_usage_audit_log_with_str_body(self):
        self._test_show_instance_usage_audit_log()

    def test_show_network_with_bytes_body_with_bytes_body(self):
        self._test_show_instance_usage_audit_log(bytes_body=True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
https://docs.openstack.org/api-ref/identity/v3/index.html#policies
"""

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class PoliciesClient(rest_client.RestClient):
    api_version = "v3"

    def create_policy(self, **kwargs):
        """Creates a Policy.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#create-policy
        """
        post_body = json.dumps({'policy': kwargs})
        resp, body = self.post('policies', post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_policies(self):
        """Lists the policies."""
        resp, body = self.get('policies')
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_policy(self, policy_id):
        """Lists out the given policy."""
        url = 'policies/%s' % policy_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_policy(self, policy_id, **kwargs):
        """Updates a policy.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-policy
        """
        post_body = json.dumps({'policy': kwargs})
        url = 'policies/%s' % policy_id
        resp, body = self.patch(url, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_policy(self, policy_id):
        """Deletes the policy."""
        url = "policies/%s" % policy_id
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_policy_association_for_endpoint(self, policy_id, endpoint_id):
        """Create policy association with endpoint.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#associate-policy-and-endpoint
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/endpoints/{1}"\
              .format(policy_id, endpoint_id)
        resp, body = self.put(url, '{}')
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_policy_association_for_endpoint(self, policy_id, endpoint_id):
        """Get policy association of endpoint.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#verify-a-policy-and-endpoint-association
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/endpoints/{1}"\
              .format(policy_id, endpoint_id)
        resp, body = self.get(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_policy_association_for_endpoint(self, policy_id, endpoint_id):
        """Delete policy association with endpoint.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#delete-a-policy-and-endpoint-association
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/endpoints/{1}"\
              .format(policy_id, endpoint_id)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_policy_association_for_service(self, policy_id, service_id):
        """Create policy association with service.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#associate-policy-and-service-type-endpoint
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/services/{1}"\
              .format(policy_id, service_id)
        resp, body = self.put(url, '{}')
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_policy_association_for_service(self, policy_id, service_id):
        """Get policy association of service.

        API Reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#verify-a-policy-and-service-type-endpoint-association
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/services/{1}"\
              .format(policy_id, service_id)
        resp, body = self.get(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_policy_association_for_service(self, policy_id, service_id):
        """Delete policy association with service.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#delete-a-policy-and-service-type-endpoint-association
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/services/{1}"\
              .format(policy_id, service_id)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_policy_association_for_region_and_service(
            self, policy_id, service_id, region_id):
        """Create policy association with service and region.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#associate-policy-and-service-type-endpoint-in-a-region
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/services/{1}/regions/{2}"\
              .format(policy_id, service_id, region_id)
        resp, body = self.put(url, '{}')
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_policy_association_for_region_and_service(
        self, policy_id, service_id, region_id):
        """Get policy association of service and region.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#verify-a-policy-and-service-type-endpoint-in-a-region-association
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/services/{1}/regions/{2}"\
              .format(policy_id, service_id, region_id)
        resp, body = self.get(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_policy_association_for_region_and_service(
        self, policy_id, service_id, region_id):
        """Delete policy association with service and region.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#delete-a-policy-and-service-type-endpoint-in-a-region-association
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/services/{1}/regions/{2}"\
              .format(policy_id, service_id, region_id)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_endpoints_for_policy(self, policy_id):
        """List policy and service endpoint associations.

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#list-policy-and-service-endpoint-associations
        """
        url = "policies/{0}/OS-ENDPOINT-POLICY/endpoints".format(policy_id)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_policy_for_endpoint(self, endpoint_id):
        """Show the effective policy associated with an endpoint

        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#show-the-effective-policy-associated-with-an-endpoint
        """
        url = "endpoints/{0}/OS-ENDPOINT-POLICY/policy".format(endpoint_id)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import manage_snapshot as schema
from tempest.lib.common import rest_client


class SnapshotManageClient(rest_client.RestClient):
    """Snapshot manage client."""

    def manage_snapshot(self, **kwargs):
        """Manage a snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#manage-an-existing-snapshot
        """
        post_body = json.dumps({'snapshot': kwargs})
        url = 'os-snapshot-manage'
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.validate_response(schema.manage_snapshot, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re
import subprocess

from tempest.tests import base


class TestTestList(base.TestCase):

    def test_stestr_list_no_errors(self):
        test_env = os.environ.copy()
        import_failures = []
        p = subprocess.Popen(['stestr', 'list'], stdout=subprocess.PIPE,
                             env=test_env)
        ids, err = p.communicate()
        self.assertEqual(0, p.returncode,
                         "test discovery failed, one or more files cause an "
                         "error on import %s" % ids)
        ids = str(ids).split('\n')
        for test_id in ids:
            if re.match(r'(\w+\.){3}\w+', test_id):
                if not test_id.startswith('tempest.'):
                    parts = test_id.partition('tempest')
                    fail_id = parts[1] + parts[2]
                    import_failures.append(fail_id)
        error_message = ("The following tests have import failures and aren't"
                         " being run with test filters %s" % import_failures)
        self.assertFalse(import_failures, error_message)



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class UserMessagesTest(base.BaseVolumeAdminTest):
    """Test volume messages with microversion greater than 3.2"""

    volume_min_microversion = '3.3'
    volume_max_microversion = 'latest'

    def _create_user_message(self):
        """Trigger a 'no valid host' situation to generate a message."""
        bad_protocol = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='storage_protocol')
        bad_vendor = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='vendor_name')
        extra_specs = {'storage_protocol': bad_protocol,
                       'vendor_name': bad_vendor}
        vol_type_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-volume-type')
        bogus_type = self.create_volume_type(
            name=vol_type_name, extra_specs=extra_specs)
        params = {'volume_type': bogus_type['id'],
                  'size': CONF.volume.volume_size}
        volume = self.create_volume(wait_until="error", **params)
        messages = self.messages_client.list_messages()['messages']
        message_id = None
        for message in messages:
            if message['resource_uuid'] == volume['id']:
                message_id = message['id']
                break
        self.assertIsNotNone(message_id, 'No user message generated for '
                                         'volume %s' % volume['id'])
        return message_id

    @decorators.idempotent_id('50f29e6e-f363-42e1-8ad1-f67ae7fd4d5a')
    def test_list_show_messages(self):
        """Test listing and showing volume messages"""
        message_id = self._create_user_message()
        self.addCleanup(self.messages_client.delete_message, message_id)

        # show message, check response schema
        self.messages_client.show_message(message_id)

        # list messages, check response schema
        self.messages_client.list_messages()

    @decorators.idempotent_id('c6eb6901-cdcc-490f-b735-4fe251842aed')
    def test_delete_message(self):
        """Test deleting volume messages"""
        message_id = self._create_user_message()
        self.messages_client.delete_message(message_id)
        self.messages_client.wait_for_resource_deletion(message_id)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import fixtures
from oslo_config import cfg

from tempest.cmd import account_generator
from tempest import config
from tempest.tests import base
from tempest.tests import fake_config


class FakeOpts(object):

    def __init__(self, version=3):
        self.os_username = 'fake_user'
        self.os_password = 'fake_password'
        self.os_project_name = 'fake_project_name'
        self.os_domain_name = 'fake_domain'
        self.tag = 'fake'
        self.concurrency = 2
        self.with_admin = True
        self.identity_version = version
        self.accounts = 'fake_accounts.yml'


class MockHelpersMixin(object):

    def mock_config_and_opts(self, identity_version):
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)
        self.opts = FakeOpts(version=identity_version)
        self.patch('oslo_log.log.setup', autospec=True)

    def mock_resource_creation(self):
        fake_resource = dict(id='id', name='name')
        self.user_create_fixture = self.useFixture(fixtures.MockPatch(
            self.cred_client + '.create_user', return_value=fake_resource))
        self.useFixture(fixtures.MockPatch(
            self.cred_client + '.create_project',
            return_value=fake_resource))
        self.useFixture(fixtures.MockPatch(
            self.cred_client + '.assign_user_role'))
        self.useFixture(fixtures.MockPatch(
            self.cred_client + '._check_role_exists',
            return_value=fake_resource))
        self.useFixture(fixtures.MockPatch(
            self.dynamic_creds + '._create_network',
            return_value=fake_resource))
        self.useFixture(fixtures.MockPatch(
            self.dynamic_creds + '._create_subnet',
            return_value=fake_resource))
        self.useFixture(fixtures.MockPatch(
            self.dynamic_creds + '._create_router',
            return_value=fake_resource))
        self.useFixture(fixtures.MockPatch(
            self.dynamic_creds + '._add_router_interface',
            return_value=fake_resource))

    def mock_domains(self):
        fake_domain_list = {'domains': [{'id': 'fake_domain',
                                         'name': 'Fake_Domain'}]}
        self.useFixture(fixtures.MockPatch(''.join([
            'tempest.lib.services.identity.v3.domains_client.'
            'DomainsClient.list_domains']),
            return_value=fake_domain_list))
        self.useFixture(fixtures.MockPatch(
            self.cred_client + '.assign_user_role_on_domain'))


class TestAccountGeneratorV2(base.TestCase, MockHelpersMixin):

    identity_version = 2

    def setUp(self):
        super(TestAccountGeneratorV2, self).setUp()
        self.mock_config_and_opts(self.identity_version)

    def test_get_credential_provider(self):
        cp = account_generator.get_credential_provider(self.opts)
        admin_creds = cp.default_admin_creds
        self.assertEqual(self.opts.tag, cp.name)
        self.assertIn(str(self.opts.identity_version), cp.identity_version)
        self.assertEqual(self.opts.os_username, admin_creds.username)
        self.assertEqual(self.opts.os_project_name, admin_creds.tenant_name)
        self.assertEqual(self.opts.os_password, admin_creds.password)
        self.assertFalse(hasattr(admin_creds, 'domain_name'))


class TestAccountGeneratorV3(TestAccountGeneratorV2):

    identity_version = 3

    def setUp(self):
        super(TestAccountGeneratorV3, self).setUp()
        fake_domain_list = {'domains': [{'id': 'fake_domain'}]}
        self.useFixture(fixtures.MockPatch(''.join([
            'tempest.lib.services.identity.v3.domains_client.'
            'DomainsClient.list_domains']),
            return_value=fake_domain_list))

    def test_get_credential_provider(self):
        cp = account_generator.get_credential_provider(self.opts)
        admin_creds = cp.default_admin_creds
        self.assertEqual(self.opts.tag, cp.name)
        self.assertIn(str(self.opts.identity_version), cp.identity_version)
        self.assertEqual(self.opts.os_username, admin_creds.username)
        self.assertEqual(self.opts.os_project_name, admin_creds.tenant_name)
        self.assertEqual(self.opts.os_password, admin_creds.password)
        self.assertEqual(self.opts.os_domain_name, admin_creds.domain_name)

    def test_get_credential_provider_without_domain(self):
        self.opts.os_domain_name = None
        cp = account_generator.get_credential_provider(self.opts)
        admin_creds = cp.default_admin_creds
        self.assertIsNotNone(admin_creds.domain_name)


class TestGenerateResourcesV2(base.TestCase, MockHelpersMixin):

    identity_version = 2
    cred_client = 'tempest.lib.common.cred_client.V2CredsClient'
    dynamic_creds = ('tempest.lib.common.dynamic_creds.'
                     'DynamicCredentialProvider')

    def setUp(self):
        super(TestGenerateResourcesV2, self).setUp()
        self.mock_config_and_opts(self.identity_version)
        self.cred_provider = account_generator.get_credential_provider(
            self.opts)
        self.mock_resource_creation()

    def test_generate_resources_no_admin(self):
        cfg.CONF.set_default('swift', False, group='service_available')
        cfg.CONF.set_default('operator_role', 'fake_operator',
                             group='object-storage')
        cfg.CONF.set_default('reseller_admin_role', 'fake_reseller',
                             group='object-storage')
        resources = account_generator.generate_resources(
            self.cred_provider, admin=False)
        resource_types = [k for k, _ in resources]
        # No admin, no swift, expect three credentials only
        self.assertEqual(3, len(resources))
        # Ensure create_user was invoked three times (three distinct users)
        self.assertEqual(3, self.user_create_fixture.mock.call_count)
        self.assertIn('primary', resource_types)
        self.assertIn('alt', resource_types)
        self.assertNotIn('admin', resource_types)
        self.assertIn(['reader'], resource_types)
        self.assertNotIn(['fake_operator'], resource_types)
        self.assertNotIn(['fake_reseller'], resource_types)
        self.assertNotIn(['fake_owner'], resource_types)
        for resource in resources:
            self.assertIsNotNone(resource[1].network)
            self.assertIsNotNone(resource[1].router)
            self.assertIsNotNone(resource[1].subnet)

    def test_generate_resources_admin(self):
        cfg.CONF.set_default('swift', False, group='service_available')
        cfg.CONF.set_default('operator_role', 'fake_operator',
                             group='object-storage')
        cfg.CONF.set_default('reseller_admin_role', 'fake_reseller',
                             group='object-storage')
        resources = account_generator.generate_resources(
            self.cred_provider, admin=True)
        resource_types = [k for k, _ in resources]
        # Admin, no swift, expect three credentials only
        self.assertEqual(4, len(resources))
        # Ensure create_user was invoked 4 times (4 distinct users)
        self.assertEqual(4, self.user_create_fixture.mock.call_count)
        self.assertIn('primary', resource_types)
        self.assertIn('alt', resource_types)
        self.assertIn('admin', resource_types)
        self.assertIn(['reader'], resource_types)
        self.assertNotIn(['fake_operator'], resource_types)
        self.assertNotIn(['fake_reseller'], resource_types)
        self.assertNotIn(['fake_owner'], resource_types)
        for resource in resources:
            self.assertIsNotNone(resource[1].network)
            self.assertIsNotNone(resource[1].router)
            self.assertIsNotNone(resource[1].subnet)

    def test_generate_resources_swift_admin(self):
        cfg.CONF.set_default('swift', True, group='service_available')
        cfg.CONF.set_default('operator_role', 'fake_operator',
                             group='object-storage')
        cfg.CONF.set_default('reseller_admin_role', 'fake_reseller',
                             group='object-storage')
        resources = account_generator.generate_resources(
            self.cred_provider, admin=True)
        resource_types = [k for k, _ in resources]
        # all options on, expect six credentials
        self.assertEqual(6, len(resources))
        # Ensure create_user was invoked 6 times (6 distinct users)
        self.assertEqual(6, self.user_create_fixture.mock.call_count)
        self.assertIn('primary', resource_types)
        self.assertIn('alt', resource_types)
        self.assertIn('admin', resource_types)
        self.assertIn(['reader'], resource_types)
        self.assertIn(['fake_operator'], resource_types)
        self.assertIn(['fake_reseller'], resource_types)
        for resource in resources:
            self.assertIsNotNone(resource[1].network)
            self.assertIsNotNone(resource[1].router)
            self.assertIsNotNone(resource[1].subnet)

    def test_generate_resources_swift_no_admin(self):
        cfg.CONF.set_default('swift', True, group='service_available')
        cfg.CONF.set_default('operator_role', 'fake_operator',
                             group='object-storage')
        cfg.CONF.set_default('reseller_admin_role', 'fake_reseller',
                             group='object-storage')
        resources = account_generator.generate_resources(
            self.cred_provider, admin=False)
        resource_types = [k for k, _ in resources]
        # No Admin, swift, expect five credentials only
        self.assertEqual(5, len(resources))
        # Ensure create_user was invoked 5 times (5 distinct users)
        self.assertEqual(5, self.user_create_fixture.mock.call_count)
        self.assertIn('primary', resource_types)
        self.assertIn('alt', resource_types)
        self.assertNotIn('admin', resource_types)
        self.assertIn(['reader'], resource_types)
        self.assertIn(['fake_operator'], resource_types)
        self.assertIn(['fake_reseller'], resource_types)
        self.assertNotIn(['fake_owner'], resource_types)
        for resource in resources:
            self.assertIsNotNone(resource[1].network)
            self.assertIsNotNone(resource[1].router)
            self.assertIsNotNone(resource[1].subnet)


class TestGenerateResourcesV3(TestGenerateResourcesV2):

    identity_version = 3
    cred_client = 'tempest.lib.common.cred_client.V3CredsClient'

    def setUp(self):
        self.mock_domains()
        super(TestGenerateResourcesV3, self).setUp()


class TestDumpAccountsV2(base.TestCase, MockHelpersMixin):

    identity_version = 2
    cred_client = 'tempest.lib.common.cred_client.V2CredsClient'
    dynamic_creds = ('tempest.lib.common.dynamic_creds.'
                     'DynamicCredentialProvider')
    domain_is_in = False

    def setUp(self):
        super(TestDumpAccountsV2, self).setUp()
        self.mock_config_and_opts(self.identity_version)
        self.cred_provider = account_generator.get_credential_provider(
            self.opts)
        self.mock_resource_creation()
        cfg.CONF.set_default('swift', True, group='service_available')
        self.resources = account_generator.generate_resources(
            self.cred_provider, admin=True)

    def test_dump_accounts(self):
        self.useFixture(fixtures.MockPatch('os.path.exists',
                                           return_value=False))
        mocked_open = mock.mock_open()
        with mock.patch('{}.open'.format(account_generator.__name__),
                        mocked_open, create=True):
            with mock.patch('yaml.safe_dump') as yaml_dump_mock:
                account_generator.setup_logging()
                account_generator.dump_accounts(self.resources,
                                                self.opts.identity_version,
                                                self.opts.accounts)
        mocked_open.assert_called_once_with(self.opts.accounts, 'w')
        handle = mocked_open()
        # Ordered args in [0], keyword args in [1]
        accounts, f = yaml_dump_mock.call_args[0]
        self.assertEqual(handle, f)
        self.assertEqual(6, len(accounts))
        if self.domain_is_in:
            self.assertIn('domain_name', accounts[0].keys())
        else:
            self.assertNotIn('domain_name', accounts[0].keys())
        self.assertEqual(1, len([x for x in accounts if
                                 x.get('types') == ['admin']]))
        self.assertEqual(3, len([x for x in accounts if 'roles' in x]))
        for account in accounts:
            self.assertIn('resources', account)
            self.assertIn('network', account.get('resources'))

    def test_dump_accounts_existing_file(self):
        self.useFixture(fixtures.MockPatch('os.path.exists',
                                           return_value=True))
        rename_mock = self.useFixture(fixtures.MockPatch('os.rename')).mock
        backup_file = '.'.join((self.opts.accounts, 'bak'))
        mocked_open = mock.mock_open()
        with mock.patch('{}.open'.format(account_generator.__name__),
                        mocked_open, create=True):
            with mock.patch('yaml.safe_dump') as yaml_dump_mock:
                account_generator.setup_logging()
                account_generator.dump_accounts(self.resources,
                                                self.opts.identity_version,
                                                self.opts.accounts)
        rename_mock.assert_called_once_with(self.opts.accounts, backup_file)
        mocked_open.assert_called_once_with(self.opts.accounts, 'w')
        handle = mocked_open()
        # Ordered args in [0], keyword args in [1]
        accounts, f = yaml_dump_mock.call_args[0]
        self.assertEqual(handle, f)
        self.assertEqual(6, len(accounts))
        if self.domain_is_in:
            self.assertIn('domain_name', accounts[0].keys())
        else:
            self.assertNotIn('domain_name', accounts[0].keys())
        self.assertEqual(1, len([x for x in accounts if
                                 x.get('types') == ['admin']]))
        self.assertEqual(3, len([x for x in accounts if 'roles' in x]))
        for account in accounts:
            self.assertIn('resources', account)
            self.assertIn('network', account.get('resources'))


class TestDumpAccountsV3(TestDumpAccountsV2):

    identity_version = 3
    cred_client = 'tempest.lib.common.cred_client.V3CredsClient'
    domain_is_in = True

    def setUp(self):
        self.mock_domains()
        super(TestDumpAccountsV3, self).setUp()


class TestAccountGeneratorCliCheck(base.TestCase):

    def setUp(self):
        super(TestAccountGeneratorCliCheck, self).setUp()
        self.account_generator = account_generator.TempestAccountGenerator(
            app=mock.Mock(), app_args=mock.Mock())
        self.parser = self.account_generator.get_parser("generator")

    def test_account_generator_zero_concurrency(self):
        error = self.assertRaises(
            SystemExit, lambda: self.parser.parse_args(
                ['-r', '0', 'accounts_file.yaml']))
        self.assertTrue(error.code != 0)

    def test_account_generator_negative_concurrency(self):
        error = self.assertRaises(
            SystemExit, lambda: self.parser.parse_args(
                ['-r', '-1', 'accounts_file.yaml']))
        self.assertTrue(error.code != 0)



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from tempest.api.compute import base
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as e

# Time that waits for until returning valid response
# TODO(takmatsu): Ideally this value would come from configuration.
VALID_WAIT = 30


class TenantUsagesTestJSON(base.BaseV2ComputeAdminTest):
    """Test tenant usages"""

    @classmethod
    def setup_clients(cls):
        super(TenantUsagesTestJSON, cls).setup_clients()
        cls.adm_client = cls.os_admin.tenant_usages_client
        cls.client = cls.os_primary.tenant_usages_client

    @classmethod
    def resource_setup(cls):
        super(TenantUsagesTestJSON, cls).resource_setup()
        cls.tenant_id = cls.client.tenant_id

        # Create a server in the demo tenant
        cls.create_test_server(wait_until='ACTIVE')

        now = datetime.datetime.now()
        cls.start = cls._parse_strtime(now - datetime.timedelta(days=1))
        cls.end = cls._parse_strtime(now + datetime.timedelta(days=1))

    @classmethod
    def _parse_strtime(cls, at):
        # Returns formatted datetime
        return at.strftime('%Y-%m-%dT%H:%M:%S.%f')

    def call_until_valid(self, func, duration, *args, **kwargs):
        # Call until get valid response for "duration"
        # because tenant usage doesn't become available immediately
        # after create VM.
        def is_valid():
            try:
                self.resp = func(*args, **kwargs)
                return True
            except e.InvalidHTTPResponseBody:
                return False
        self.assertEqual(test_utils.call_until_true(is_valid, duration, 1),
                         True, "%s not return valid response in %s secs" % (
                             func.__name__, duration))
        return self.resp

    @decorators.idempotent_id('062c8ae9-9912-4249-8b51-e38d664e926e')
    def test_list_usage_all_tenants(self):
        """Test getting usage for all tenants"""
        tenant_usage = self.call_until_valid(
            self.adm_client.list_tenant_usages, VALID_WAIT,
            start=self.start, end=self.end, detailed="1")['tenant_usages'][0]
        self.assertEqual(len(tenant_usage), 8)

    @decorators.idempotent_id('94135049-a4c5-4934-ad39-08fa7da4f22e')
    def test_get_usage_tenant(self):
        """Test getting usage for a specific tenant"""
        tenant_usage = self.call_until_valid(
            self.adm_client.show_tenant_usage, VALID_WAIT,
            self.tenant_id, start=self.start, end=self.end)['tenant_usage']

        self.assertEqual(len(tenant_usage), 8)

    @decorators.idempotent_id('9d00a412-b40e-4fd9-8eba-97b496316116')
    def test_get_usage_tenant_with_non_admin_user(self):
        """Test getting usage for a specific tenant with non admin user"""
        tenant_usage = self.call_until_valid(
            self.client.show_tenant_usage, VALID_WAIT,
            self.tenant_id, start=self.start, end=self.end)['tenant_usage']
        self.assertEqual(len(tenant_usage), 8)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

node = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'interfaces': {'type': 'array'},
        'host': {'type': 'string'},
        'task_state': {'type': ['string', 'null']},
        'cpus': {'type': ['integer', 'string']},
        'memory_mb': {'type': ['integer', 'string']},
        'disk_gb': {'type': ['integer', 'string']},
    },
    'additionalProperties': False,
    'required': ['id', 'interfaces', 'host', 'task_state', 'cpus', 'memory_mb',
                 'disk_gb']
}

list_baremetal_nodes = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'nodes': {
                'type': 'array',
                'items': node
            }
        },
        'additionalProperties': False,
        'required': ['nodes']
    }
}

baremetal_node = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'node': node
        },
        'additionalProperties': False,
        'required': ['node']
    }
}
get_baremetal_node = copy.deepcopy(baremetal_node)
get_baremetal_node['response_body']['properties']['node'][
    'properties'].update({'instance_uuid': {'type': ['string', 'null']}})
get_baremetal_node['response_body']['properties']['node'][
    'required'].append('instance_uuid')



# Copyright 2014 NEC Technologies India Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ServerGroupTestJSON(base.BaseV2ComputeTest):
    """These tests check for the server-group APIs.

    They create/delete server-groups with different policies.
    policies = affinity/anti-affinity
    It also adds the tests for list and get details of server-groups
    """
    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(ServerGroupTestJSON, cls).setup_clients()
        cls.client = cls.server_groups_client

    @classmethod
    def _set_policy(cls, policy):
        if not cls.is_requested_microversion_compatible('2.63'):
            return policy[0]
        else:
            return policy

    @classmethod
    def resource_setup(cls):
        super(ServerGroupTestJSON, cls).resource_setup()
        if cls.is_requested_microversion_compatible('2.63'):
            cls.policy_field = 'policies'
            cls.policy = ['affinity']
        else:
            cls.policy_field = 'policy'
            cls.policy = 'affinity'

    def setUp(self):
        super(ServerGroupTestJSON, self).setUp()
        # TODO(zhufl): After microversion 2.13 project_id and user_id are
        # added to the body of server_group, and microversion is not used
        # in resource_setup for now, so we should create server group in setUp
        # in order to use the same microversion as in testcases till
        # microversion support in resource_setup is fulfilled.
        if not hasattr(self, 'created_server_group'):
            self.__class__.created_server_group = \
                self.create_test_server_group(policy=self.policy)

    def _create_server_group(self, name, policy):
        # create the test server-group with given policy
        server_group = {'name': name, self.policy_field: policy}
        body = self.create_test_server_group(name, policy)
        for key in ['name', self.policy_field]:
            self.assertEqual(server_group[key], body[key])
        return body

    def _delete_server_group(self, server_group):
        # delete the test server-group
        self.client.delete_server_group(server_group['id'])
        # validation of server-group deletion
        server_group_list = self.client.list_server_groups()['server_groups']
        self.assertNotIn(server_group, server_group_list)

    def _create_delete_server_group(self, policy):
        # Create and Delete the server-group with given policy
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='server-group')
        server_group = self._create_server_group(name, policy)
        self._delete_server_group(server_group)

    @decorators.idempotent_id('5dc57eda-35b7-4af7-9e5f-3c2be3d2d68b')
    def test_create_delete_server_group_with_affinity_policy(self):
        """Test Create/Delete the server-group with affinity policy"""
        self._create_delete_server_group(self.policy)

    @decorators.idempotent_id('3645a102-372f-4140-afad-13698d850d23')
    def test_create_delete_server_group_with_anti_affinity_policy(self):
        """Test Create/Delete the server-group with anti-affinity policy"""
        policy = self._set_policy(['anti-affinity'])
        self._create_delete_server_group(policy)

    @decorators.idempotent_id('154dc5a4-a2fe-44b5-b99e-f15806a4a113')
    def test_create_delete_multiple_server_groups_with_same_name_policy(self):
        """Test Create/Delete the server-groups with same name and policy"""
        server_groups = []
        server_group_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='server-group')
        for _ in range(0, 2):
            server_groups.append(self._create_server_group(server_group_name,
                                                           self.policy))
        for key in ['name', self.policy_field]:
            self.assertEqual(server_groups[0][key], server_groups[1][key])
        self.assertNotEqual(server_groups[0]['id'], server_groups[1]['id'])

        for i in range(0, 2):
            self._delete_server_group(server_groups[i])

    @decorators.idempotent_id('b3545034-dd78-48f0-bdc2-a4adfa6d0ead')
    def test_show_server_group(self):
        """Test getting the server-group detail"""
        body = self.client.show_server_group(
            self.created_server_group['id'])['server_group']
        self.assertEqual(self.created_server_group, body)

    @decorators.idempotent_id('d4874179-27b4-4d7d-80e4-6c560cdfe321')
    def test_list_server_groups(self):
        """Test listing the server-groups"""
        body = self.client.list_server_groups()['server_groups']
        self.assertIn(self.created_server_group, body)

    @decorators.idempotent_id('ed20d3fb-9d1f-4329-b160-543fbd5d9811')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAffinityFilter"),
        'ServerGroupAffinityFilter is not available.')
    def test_create_server_with_scheduler_hint_group(self):
        """Test creating a server with the scheduler hint 'group'"""
        hints = {'group': self.created_server_group['id']}
        server = self.create_test_server(scheduler_hints=hints,
                                         wait_until='ACTIVE')
        self.addCleanup(self.delete_server, server['id'])

        # Check a server is in the group
        server_group = (self.server_groups_client.show_server_group(
            self.created_server_group['id'])['server_group'])
        self.assertIn(server['id'], server_group['members'])


class ServerGroup264TestJSON(base.BaseV2ComputeTest):
    """These tests check for the server-group APIs 2.64 microversion.

    This tests is only to verify the POST, GET server-groups APIs response
    schema with 2.64 microversion
    """
    create_default_network = True
    min_microversion = '2.64'

    @decorators.idempotent_id('b52f09dd-2133-4037-9a5d-bdb260096a88')
    def test_create_get_server_group(self):
        # create, get the test server-group with given policy
        server_group = self.create_test_server_group(
            name='server-group', policy='affinity')
        self.addCleanup(
            self.server_groups_client.delete_server_group,
            server_group['id'])
        self.server_groups_client.list_server_groups()
        self.server_groups_client.show_server_group(server_group['id'])



# Copyright 2018 SUSE Linux GmbH
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib import decorators


class ApplicationCredentialsV3AdminTest(base.BaseApplicationCredentialsV3Test,
                                        base.BaseIdentityV3AdminTest):
    """Test keystone application credentials"""

    @decorators.idempotent_id('3b3dd48f-3388-406a-a9e6-4d078a552d0e')
    def test_create_application_credential_with_roles(self):
        """Test creating keystone application credential with roles"""
        role = self.setup_test_role()
        self.os_admin.roles_v3_client.create_user_role_on_project(
            self.project_id,
            self.user_id,
            role['id']
        )

        app_cred = self.create_application_credential(
            roles=[{'id': role['id']}])
        secret = app_cred['secret']

        # Check that the application credential is functional
        _, resp = self.non_admin_token.get_token(
            app_cred_id=app_cred['id'],
            app_cred_secret=secret,
            auth_data=True
        )
        self.assertEqual(resp['project']['id'], self.project_id)
        self.assertEqual(resp['roles'][0]['id'], role['id'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# The 2.45 microversion removes the "location" header and adds "image_id"
# to the response body.
create_image = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'image_id': {'type': 'string'}
        },
        'additionalProperties': False,
        'required': ['image_id']
    }
}

# NOTE(mriedem): The compute proxy APIs for showing/listing and deleting
# images were deprecated in microversion 2.35, and the compute proxy APIs for
# working with image metadata were deprecated in microversion 2.39. Therefore,
# client-side code shouldn't rely on those APIs in the compute images client
# past those microversions and should instead use the Glance images client
# directly.



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class VolumeTypesTest(base.BaseVolumeAdminTest):
    """Test volume types"""

    @decorators.idempotent_id('9d9b28e3-1b2e-4483-a2cc-24aa0ea1de54')
    def test_volume_type_list(self):
        """Test listing volume types"""
        body = \
            self.admin_volume_types_client.list_volume_types()['volume_types']
        self.assertIsInstance(body, list)

    @decorators.idempotent_id('c03cc62c-f4e9-4623-91ec-64ce2f9c1260')
    def test_volume_crud_with_volume_type_and_extra_specs(self):
        """Test create/update/get/delete volume with volume_type"""
        volume_types = list()
        vol_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-volume')
        proto = CONF.volume.storage_protocol
        vendor = CONF.volume.vendor_name
        extra_specs = {"storage_protocol": proto,
                       "vendor_name": vendor}
        # Create two volume_types
        for _ in range(2):
            vol_type = self.create_volume_type(
                extra_specs=extra_specs)
            volume_types.append(vol_type)
        params = {'name': vol_name,
                  'volume_type': volume_types[0]['id'],
                  'size': CONF.volume.volume_size}

        # Create volume
        volume = self.create_volume(**params)
        self.assertEqual(volume_types[0]['name'], volume["volume_type"])
        self.assertEqual(volume['name'], vol_name,
                         "The created volume name is not equal "
                         "to the requested name")
        self.assertIsNotNone(volume['id'],
                             "Field volume id is empty or not found.")

        # Update volume with new volume_type
        self.volumes_client.retype_volume(volume['id'],
                                          new_type=volume_types[1]['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        # Get volume details and Verify
        fetched_volume = self.volumes_client.show_volume(
            volume['id'])['volume']
        self.assertEqual(volume_types[1]['name'],
                         fetched_volume['volume_type'],
                         'The fetched Volume type is different '
                         'from updated volume type')
        self.assertEqual(vol_name, fetched_volume['name'],
                         'The fetched Volume is different '
                         'from the created Volume')
        self.assertEqual(volume['id'], fetched_volume['id'],
                         'The fetched Volume is different '
                         'from the created Volume')

    @decorators.idempotent_id('4e955c3b-49db-4515-9590-0c99f8e471ad')
    def test_volume_type_create_get_delete(self):
        """Test create/get/delete volume type"""
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-volume-type')
        description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="volume-type-description")
        proto = CONF.volume.storage_protocol
        vendor = CONF.volume.vendor_name
        extra_specs = {"storage_protocol": proto,
                       "vendor_name": vendor}
        params = {'name': name,
                  'description': description,
                  'extra_specs': extra_specs,
                  'os-volume-type-access:is_public': True}
        body = self.create_volume_type(**params)
        self.assertEqual(name, body['name'],
                         "The created volume_type name is not equal "
                         "to the requested name")
        self.assertEqual(description, body['description'],
                         "The created volume_type_description name is "
                         "not equal to the requested name")
        fetched_volume_type = self.admin_volume_types_client.show_volume_type(
            body['id'])['volume_type']
        self.assertEqual(name, fetched_volume_type['name'],
                         'The fetched Volume_type is different '
                         'from the created Volume_type')
        self.assertEqual(str(body['id']), fetched_volume_type['id'],
                         'The fetched Volume_type is different '
                         'from the created Volume_type')
        self.assertEqual(extra_specs, fetched_volume_type['extra_specs'],
                         'The fetched Volume_type is different '
                         'from the created Volume_type')
        self.assertEqual(description, fetched_volume_type['description'])
        self.assertEqual(body['is_public'],
                         fetched_volume_type['is_public'])
        self.assertEqual(
            body['os-volume-type-access:is_public'],
            fetched_volume_type['os-volume-type-access:is_public'])

    @decorators.idempotent_id('7830abd0-ff99-4793-a265-405684a54d46')
    def test_volume_type_encryption_create_get_update_delete(self):
        """Test create/get/update/delete volume encryption type"""
        create_kwargs = {'provider': 'LuksEncryptor',
                         'key_size': 256,
                         'cipher': 'aes-xts-plain64',
                         'control_location': 'front-end'}
        volume_type_id = self.create_volume_type()['id']

        # Create encryption type
        encryption_type = \
            self.admin_encryption_types_client.create_encryption_type(
                volume_type_id, **create_kwargs)['encryption']
        for key in create_kwargs:
            self.assertEqual(create_kwargs[key], encryption_type[key],
                             'The created encryption_type %s is different '
                             'from the requested encryption_type' % key)

        # Get encryption type
        encrypt_type_id = encryption_type['volume_type_id']
        encryption_id = encryption_type['encryption_id']
        fetched_encryption_type = (
            self.admin_encryption_types_client.show_encryption_type(
                encrypt_type_id))
        for key in create_kwargs:
            self.assertEqual(create_kwargs[key], fetched_encryption_type[key],
                             'The fetched encryption_type %s is different '
                             'from the created encryption_type' % key)

        # Update encryption type
        update_kwargs = {'key_size': 128,
                         'provider': 'SomeProvider',
                         'cipher': 'aes-xts-plain64',
                         'control_location': 'back-end'}
        self.admin_encryption_types_client.update_encryption_type(
            encrypt_type_id, encryption_id, **update_kwargs)
        updated_encryption_type = (
            self.admin_encryption_types_client.show_encryption_type(
                encrypt_type_id))
        for key in update_kwargs:
            self.assertEqual(update_kwargs[key], updated_encryption_type[key],
                             'The fetched encryption_type %s is different '
                             'from the updated encryption_type' % key)

        # Get encryption specs item
        key = 'cipher'
        item = self.admin_encryption_types_client.show_encryption_specs_item(
            encrypt_type_id, key)
        self.assertEqual(update_kwargs[key], item[key])

        # Delete encryption type
        self.admin_encryption_types_client.delete_encryption_type(
            encrypt_type_id, encryption_id)
        self.admin_encryption_types_client.wait_for_resource_deletion(
            encrypt_type_id)
        deleted_encryption_type = (
            self.admin_encryption_types_client.show_encryption_type(
                encrypt_type_id))
        self.assertEmpty(deleted_encryption_type)

    @decorators.idempotent_id('cf9f07c6-db9e-4462-a243-5933ad65e9c8')
    def test_volume_type_update(self):
        """Test updating volume type details"""
        # Create volume type
        volume_type = self.create_volume_type()

        # New volume type details
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="volume-type")
        description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="volume-type-description")
        is_public = not volume_type['is_public']

        # Update volume type details
        kwargs = {'name': name,
                  'description': description,
                  'is_public': is_public}
        updated_vol_type = self.admin_volume_types_client.update_volume_type(
            volume_type['id'], **kwargs)['volume_type']
        self.assertEqual(name, updated_vol_type['name'])
        self.assertEqual(description, updated_vol_type['description'])
        self.assertEqual(is_public, updated_vol_type['is_public'])

        # Verify volume type details were updated
        fetched_volume_type = self.admin_volume_types_client.show_volume_type(
            volume_type['id'])['volume_type']
        self.assertEqual(name, fetched_volume_type['name'])
        self.assertEqual(description, fetched_volume_type['description'])
        self.assertEqual(is_public, fetched_volume_type['is_public'])



# Copyright (c) 2019 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class QosMinimumBandwidthRulesClient(base.BaseNetworkClient):

    def create_minimum_bandwidth_rule(self, qos_policy_id, **kwargs):
        """Creates a minimum bandwidth rule for a QoS policy.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-minimum-bandwidth-rule
        """
        uri = '/qos/policies/%s/minimum_bandwidth_rules' % qos_policy_id
        post_data = {'minimum_bandwidth_rule': kwargs}
        return self.create_resource(uri, post_data)

    def update_minimum_bandwidth_rule(self, qos_policy_id, rule_id, **kwargs):
        """Updates a minimum bandwidth rule.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-minimum-bandwidth-rule
        """
        uri = '/qos/policies/%s/minimum_bandwidth_rules/%s' % (
            qos_policy_id, rule_id)
        post_data = {'minimum_bandwidth_rule': kwargs}
        return self.update_resource(uri, post_data)

    def show_minimum_bandwidth_rule(self, qos_policy_id, rule_id, **fields):
        """Show details of a minimum bandwidth rule.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-minimum-bandwidth-rule-details
        """
        uri = '/qos/policies/%s/minimum_bandwidth_rules/%s' % (
            qos_policy_id, rule_id)
        return self.show_resource(uri, **fields)

    def delete_minimum_bandwidth_rule(self, qos_policy_id, rule_id):
        """Deletes a minimum bandwidth rule for a QoS policy.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-minimum-bandwidth-rule
        """
        uri = '/qos/policies/%s/minimum_bandwidth_rules/%s' % (
            qos_policy_id, rule_id)
        return self.delete_resource(uri)

    def list_minimum_bandwidth_rules(self, qos_policy_id, **filters):
        """Lists all minimum bandwidth rules for a QoS policy.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-minimum-bandwidth-rules-for-qos-policy
        """
        uri = '/qos/policies/%s/minimum_bandwidth_rules' % qos_policy_id
        return self.list_resources(uri, **filters)



# Copyright 2022 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.volume import transfers

# Volume microversion 3.55:
# Add 'no_snapshots' attribute in 'transfer' responses.

create_volume_transfer = copy.deepcopy(transfers.create_volume_transfer)
create_volume_transfer['response_body']['properties']['transfer'][
    'properties'].update({'no_snapshots': {'type': 'boolean'}})

common_show_volume_transfer = copy.deepcopy(
    transfers.common_show_volume_transfer)
common_show_volume_transfer['properties'].update(
    {'no_snapshots': {'type': 'boolean'}})

show_volume_transfer = copy.deepcopy(transfers.show_volume_transfer)
show_volume_transfer['response_body']['properties'][
    'transfer'] = common_show_volume_transfer

list_volume_transfers_no_detail = copy.deepcopy(
    transfers.list_volume_transfers_no_detail)

list_volume_transfers_with_detail = copy.deepcopy(
    transfers.list_volume_transfers_with_detail)
list_volume_transfers_with_detail['response_body']['properties']['transfers'][
    'items'] = common_show_volume_transfer

delete_volume_transfer = copy.deepcopy(transfers.delete_volume_transfer)

accept_volume_transfer = copy.deepcopy(transfers.accept_volume_transfer)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import shlex
import subprocess

from oslo_log import log as logging

from tempest.lib import base
import tempest.lib.cli.output_parser
from tempest.lib import exceptions


LOG = logging.getLogger(__name__)


def execute(cmd, action, flags='', params='', fail_ok=False,
            merge_stderr=False, cli_dir='/usr/bin', prefix=''):
    """Executes specified command for the given action.

    :param cmd: command to be executed
    :type cmd: string
    :param action: string of the cli command to run
    :type action: string
    :param flags: any optional cli flags to use
    :type flags: string
    :param params: string of any optional positional args to use
    :type params: string
    :param fail_ok: boolean if True an exception is not raised when the
                    cli return code is non-zero
    :type fail_ok: boolean
    :param merge_stderr: boolean if True the stderr buffer is merged into
                         stdout
    :type merge_stderr: boolean
    :param cli_dir: The path where the cmd can be executed
    :type cli_dir: string
    :param prefix: prefix to insert before command
    :type prefix: string
    """
    cmd = ' '.join([prefix, os.path.join(cli_dir, cmd),
                    flags, action, params])
    cmd = cmd.strip()
    LOG.info("running: '%s'", cmd)
    cmd = shlex.split(cmd)
    stdout = subprocess.PIPE
    stderr = subprocess.STDOUT if merge_stderr else subprocess.PIPE
    proc = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
    result, result_err = proc.communicate()
    if not fail_ok and proc.returncode != 0:
        raise exceptions.CommandFailed(proc.returncode,
                                       cmd,
                                       result,
                                       result_err)
    return os.fsdecode(result)


class CLIClient(object):
    """Class to use OpenStack official python client CLI's with auth

    :param username: The username to authenticate with
    :type username: string
    :param password: The password to authenticate with
    :type password: string
    :param tenant_name: The name of the tenant to use with the client calls
    :type tenant_name: string
    :param uri: The auth uri for the OpenStack Deployment
    :type uri: string
    :param cli_dir: The path where the python client binaries are installed.
                    defaults to /usr/bin
    :type cli_dir: string
    :param insecure: if True, --insecure is passed to python client binaries.
    :type insecure: boolean
    :param prefix: prefix to insert before commands
    :type prefix: string
    :param user_domain_name: User's domain name
    :type user_domain_name: string
    :param user_domain_id: User's domain ID
    :type user_domain_id: string
    :param project_domain_name: Project's domain name
    :type project_domain_name: string
    :param project_domain_id: Project's domain ID
    :type project_domain_id: string
    :param identity_api_version: Version of the Identity API
    :type identity_api_version: string
    """

    CLIENTS_WITHOUT_IDENTITY_VERSION = ['nova', 'nova_manage', 'keystone',
                                        'glance', 'ceilometer', 'heat',
                                        'cinder', 'neutron', 'sahara']

    def __init__(self, username='', password='', tenant_name='', uri='',
                 cli_dir='', insecure=False, prefix='', user_domain_name=None,
                 user_domain_id=None, project_domain_name=None,
                 project_domain_id=None, identity_api_version=None, *args,
                 **kwargs):
        """Initialize a new CLIClient object."""
        super(CLIClient, self).__init__()
        self.cli_dir = cli_dir if cli_dir else '/usr/bin'
        self.username = username
        self.tenant_name = tenant_name
        self.password = password
        self.uri = uri
        self.insecure = insecure
        self.prefix = prefix
        self.user_domain_name = user_domain_name
        self.user_domain_id = user_domain_id
        self.project_domain_name = project_domain_name
        self.project_domain_id = project_domain_id
        self.identity_api_version = identity_api_version

    def nova(self, action, flags='', params='', fail_ok=False,
             endpoint_type='publicURL', merge_stderr=False):
        """Executes nova command for the given action.

        :param action: the cli command to run using nova
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --os-endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'nova', action, flags, params, fail_ok, merge_stderr)

    def nova_manage(self, action, flags='', params='', fail_ok=False,
                    merge_stderr=False):
        """Executes nova-manage command for the given action.

        :param action: the cli command to run using nova-manage
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        return execute(
            'nova-manage', action, flags, params, fail_ok, merge_stderr,
            self.cli_dir)

    def keystone(self, action, flags='', params='', fail_ok=False,
                 merge_stderr=False):
        """Executes keystone command for the given action.

        :param action: the cli command to run using keystone
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        return self.cmd_with_auth(
            'keystone', action, flags, params, fail_ok, merge_stderr)

    def glance(self, action, flags='', params='', fail_ok=False,
               endpoint_type='publicURL', merge_stderr=False):
        """Executes glance command for the given action.

        :param action: the cli command to run using glance
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --os-endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'glance', action, flags, params, fail_ok, merge_stderr)

    def ceilometer(self, action, flags='', params='',
                   fail_ok=False, endpoint_type='publicURL',
                   merge_stderr=False):
        """Executes ceilometer command for the given action.

        :param action: the cli command to run using ceilometer
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --os-endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'ceilometer', action, flags, params, fail_ok, merge_stderr)

    def heat(self, action, flags='', params='',
             fail_ok=False, endpoint_type='publicURL', merge_stderr=False):
        """Executes heat command for the given action.

        :param action: the cli command to run using heat
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --os-endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'heat', action, flags, params, fail_ok, merge_stderr)

    def cinder(self, action, flags='', params='', fail_ok=False,
               endpoint_type='publicURL', merge_stderr=False):
        """Executes cinder command for the given action.

        :param action: the cli command to run using cinder
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --os-endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'cinder', action, flags, params, fail_ok, merge_stderr)

    def swift(self, action, flags='', params='', fail_ok=False,
              endpoint_type='publicURL', merge_stderr=False):
        """Executes swift command for the given action.

        :param action: the cli command to run using swift
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --os-endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'swift', action, flags, params, fail_ok, merge_stderr)

    def neutron(self, action, flags='', params='', fail_ok=False,
                endpoint_type='publicURL', merge_stderr=False):
        """Executes neutron command for the given action.

        :param action: the cli command to run using neutron
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'neutron', action, flags, params, fail_ok, merge_stderr)

    def sahara(self, action, flags='', params='',
               fail_ok=False, endpoint_type='publicURL', merge_stderr=True):
        """Executes sahara command for the given action.

        :param action: the cli command to run using sahara
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param endpoint_type: the type of endpoint for the service
        :type endpoint_type: string
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        flags += ' --endpoint-type %s' % endpoint_type
        return self.cmd_with_auth(
            'sahara', action, flags, params, fail_ok, merge_stderr)

    def openstack(self, action, flags='', params='', fail_ok=False,
                  merge_stderr=False):
        """Executes openstack command for the given action.

        :param action: the cli command to run using openstack
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the
                        cli return code is non-zero
        :type fail_ok: boolean
        :param merge_stderr: if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        return self.cmd_with_auth(
            'openstack', action, flags, params, fail_ok, merge_stderr)

    def cmd_with_auth(self, cmd, action, flags='', params='',
                      fail_ok=False, merge_stderr=False):
        """Executes given command with auth attributes appended.

        :param cmd: command to be executed
        :type cmd: string
        :param action: command on cli to run
        :type action: string
        :param flags: optional cli flags to use
        :type flags: string
        :param params: optional positional args to use
        :type params: string
        :param fail_ok: if True an exception is not raised when the cli return
                        code is non-zero
        :type fail_ok: boolean
        :param merge_stderr:  if True the stderr buffer is merged into stdout
        :type merge_stderr: boolean
        """
        creds = ('--os-username %s --os-password %s '
                 '--os-auth-url %s' %
                 (self.username,
                  self.password,
                  self.uri))
        if self.tenant_name is not None:
            creds += ' --os-project-name %s' % self.tenant_name
        if self.identity_api_version:
            if cmd not in self.CLIENTS_WITHOUT_IDENTITY_VERSION:
                creds += ' --os-identity-api-version %s' % (
                    self.identity_api_version)
        if self.user_domain_name is not None:
            creds += ' --os-user-domain-name %s' % self.user_domain_name
        if self.user_domain_id is not None:
            creds += ' --os-user-domain-id %s' % self.user_domain_id
        if self.project_domain_name is not None:
            creds += ' --os-project-domain-name %s' % self.project_domain_name
        if self.project_domain_id is not None:
            creds += ' --os-project-domain-id %s' % self.project_domain_id
        if self.insecure:
            flags = creds + ' --insecure ' + flags
        else:
            flags = creds + ' ' + flags
        return execute(cmd, action, flags, params, fail_ok, merge_stderr,
                       self.cli_dir, prefix=self.prefix)


class ClientTestBase(base.BaseTestCase):
    """Base test class for testing the OpenStack client CLI interfaces."""

    def setUp(self):
        super(ClientTestBase, self).setUp()
        self.clients = self._get_clients()
        self.parser = tempest.lib.cli.output_parser

    def _get_clients(self):
        """Abstract method to initialize CLIClient object.

        This method must be overloaded in child test classes. It should be
        used to initialize the CLIClient object with the appropriate
        credentials during the setUp() phase of tests.
        """
        raise NotImplementedError

    def assertTableStruct(self, items, field_names):
        """Verify that all items has keys listed in field_names.

        :param items: items to assert are field names in the output table
        :type items: list
        :param field_names: field names from the output table of the cmd
        :type field_names: list
        """
        for item in items:
            for field in field_names:
                self.assertIn(field, item)

    def assertFirstLineStartsWith(self, lines, beginning):
        """Verify that the first line starts with a string

        :param lines: strings for each line of output
        :type lines: list
        :param beginning: verify this is at the beginning of the first line
        :type beginning: string
        """
        self.assertTrue(lines[0].startswith(beginning),
                        msg=('Beginning of first line has invalid content: %s'
                             % lines[:3]))



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class ServersOnMultiNodesTest(base.BaseV2ComputeAdminTest):
    """Test creating servers on multiple nodes with scheduler_hints."""
    @classmethod
    def resource_setup(cls):
        super(ServersOnMultiNodesTest, cls).resource_setup()
        cls.server01 = cls.create_test_server(wait_until='ACTIVE')['id']
        cls.host01 = cls.get_host_for_server(cls.server01)

    @classmethod
    def skip_checks(cls):
        super(ServersOnMultiNodesTest, cls).skip_checks()

        if CONF.compute.min_compute_nodes < 2:
            raise cls.skipException(
                "Less than 2 compute nodes, skipping multi-nodes test.")

    def _create_servers_with_group(self, policy):
        group_id = self.create_test_server_group(policy=[policy])['id']
        hints = {'group': group_id}
        reservation_id = self.create_test_server(
            scheduler_hints=hints, wait_until='ACTIVE', min_count=2,
            return_reservation_id=True)['reservation_id']

        # Get the servers using the reservation_id.
        servers = self.servers_client.list_servers(
            detail=True, reservation_id=reservation_id)['servers']
        self.assertEqual(2, len(servers))

        # Assert the servers are in the group.
        server_group = self.server_groups_client.show_server_group(
            group_id)['server_group']
        hosts = {}
        for server in servers:
            self.assertIn(server['id'], server_group['members'])
            hosts[server['id']] = self.get_host_for_server(server['id'])

        return hosts

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('26a9d5df-6890-45f2-abc4-a659290cb130')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("SameHostFilter"),
        'SameHostFilter is not available.')
    def test_create_servers_on_same_host(self):
        """Test creating servers with hints 'same_host'"""
        hints = {'same_host': self.server01}
        server02 = self.create_test_server(scheduler_hints=hints,
                                           wait_until='ACTIVE')['id']
        host02 = self.get_host_for_server(server02)
        self.assertEqual(self.host01, host02)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('cc7ca884-6e3e-42a3-a92f-c522fcf25e8e')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("DifferentHostFilter"),
        'DifferentHostFilter is not available.')
    def test_create_servers_on_different_hosts(self):
        """Test creating servers with hints of single 'different_host'"""
        hints = {'different_host': self.server01}
        server02 = self.create_test_server(scheduler_hints=hints,
                                           wait_until='ACTIVE')['id']
        host02 = self.get_host_for_server(server02)
        self.assertNotEqual(self.host01, host02)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('7869cc84-d661-4e14-9f00-c18cdc89cf57')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("DifferentHostFilter"),
        'DifferentHostFilter is not available.')
    def test_create_servers_on_different_hosts_with_list_of_servers(self):
        """Test creating servers with hints of a list of 'different_host'"""
        hints = {'different_host': [self.server01]}
        server02 = self.create_test_server(scheduler_hints=hints,
                                           wait_until='ACTIVE')['id']
        host02 = self.get_host_for_server(server02)
        self.assertNotEqual(self.host01, host02)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('f8bd0867-e459-45f5-ba53-59134552fe04')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAntiAffinityFilter"),
        'ServerGroupAntiAffinityFilter is not available.')
    def test_create_server_with_scheduler_hint_group_anti_affinity(self):
        """Tests the ServerGroupAntiAffinityFilter

        Creates two servers in an anti-affinity server group and
        asserts the servers are in the group and on different hosts.

        Uses the /servers multi-create API.
        """
        hosts = self._create_servers_with_group('anti-affinity')
        hostnames = list(hosts.values())
        self.assertNotEqual(hostnames[0], hostnames[1],
                            'Servers are on the same host: %s' % hosts)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('9d2e924a-baf4-11e7-b856-fa163e65f5ce')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAffinityFilter"),
        'ServerGroupAffinityFilter is not available.')
    def test_create_server_with_scheduler_hint_group_affinity(self):
        """Tests the ServerGroupAffinityFilter

        Creates two servers in an affinity server group and
        asserts the servers are in the group and on same host.

        Uses the /servers multi-create API.
        """
        hosts = self._create_servers_with_group('affinity')
        hostnames = list(hosts.values())
        self.assertEqual(hostnames[0], hostnames[1],
                         'Servers are on the different hosts: %s' % hosts)


class UnshelveToHostMultiNodesTest(base.BaseV2ComputeAdminTest):
    """Test to unshelve server in between hosts."""
    min_microversion = '2.91'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(UnshelveToHostMultiNodesTest, cls).skip_checks()

        if CONF.compute.min_compute_nodes < 2:
            raise cls.skipException(
                "Less than 2 compute nodes, skipping multi-nodes test.")

    def _shelve_offload_then_unshelve_to_host(self, server, host):
        compute.shelve_server(self.servers_client, server['id'],
                              force_shelve_offload=True)

        # Work around https://bugs.launchpad.net/nova/+bug/2045785
        # This can be removed when ^ is fixed.
        def _check_server_host_is_none():
            server_details = self.os_admin.servers_client.show_server(
                server['id'])
            self.assertIsNone(server_details['server']['OS-EXT-SRV-ATTR:host'])

        self.wait_for(_check_server_host_is_none)

        self.os_admin.servers_client.unshelve_server(
            server['id'],
            body={'unshelve': {'host': host}}
            )
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('b5cc0889-50c2-46a0-b8ff-b5fb4c3a6e20')
    def test_unshelve_to_specific_host(self):
        """Test unshelve to a specific host, new behavior introduced in
        microversion 2.91.
        1. Shelve offload server.
        2. Request unshelve to original host and verify server land on it.
        3. Shelve offload server again.
        4. Request unshelve to the other host and verify server land on it.
        """
        server = self.create_test_server(wait_until='ACTIVE')
        host = self.get_host_for_server(server['id'])
        otherhost = self.get_host_other_than(server['id'])

        self._shelve_offload_then_unshelve_to_host(server, host)
        self.assertEqual(host, self.get_host_for_server(server['id']))

        self._shelve_offload_then_unshelve_to_host(server, otherhost)
        self.assertEqual(otherhost, self.get_host_for_server(server['id']))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class VolumeTypesExtraSpecsTest(base.BaseVolumeAdminTest):
    """Test volume type extra specs"""

    @classmethod
    def resource_setup(cls):
        super(VolumeTypesExtraSpecsTest, cls).resource_setup()
        cls.volume_type = cls.create_volume_type()

    @decorators.idempotent_id('b42923e9-0452-4945-be5b-d362ae533e60')
    def test_volume_type_extra_specs_list(self):
        """Test listing volume type extra specs"""
        extra_specs = {"spec1": "val1"}
        body = self.admin_volume_types_client.create_volume_type_extra_specs(
            self.volume_type['id'], extra_specs)['extra_specs']
        self.assertEqual(extra_specs, body,
                         "Volume type extra spec incorrectly created")
        body = self.admin_volume_types_client.list_volume_types_extra_specs(
            self.volume_type['id'])['extra_specs']
        self.assertIsInstance(body, dict)
        self.assertIn('spec1', body)

    @decorators.idempotent_id('0806db36-b4a0-47a1-b6f3-c2e7f194d017')
    def test_volume_type_extra_specs_update(self):
        """Test updating volume type extra specs"""
        extra_specs = {"spec2": "val1"}
        body = self.admin_volume_types_client.create_volume_type_extra_specs(
            self.volume_type['id'], extra_specs)['extra_specs']
        self.assertEqual(extra_specs, body,
                         "Volume type extra spec incorrectly created")

        # Only update an extra spec
        spec_key = "spec2"
        extra_spec = {spec_key: "val2"}
        body = self.admin_volume_types_client.update_volume_type_extra_specs(
            self.volume_type['id'], spec_key, extra_spec)
        self.assertIn(spec_key, body)
        self.assertEqual(extra_spec[spec_key], body[spec_key])
        body = self.admin_volume_types_client.show_volume_type_extra_specs(
            self.volume_type['id'], spec_key)
        self.assertIn(spec_key, body)
        self.assertEqual(extra_spec[spec_key], body[spec_key],
                         "Volume type extra spec incorrectly updated")

        # Update an existing extra spec and create a new extra spec
        extra_specs = {spec_key: "val3", "spec4": "val4"}
        body = self.admin_volume_types_client.create_volume_type_extra_specs(
            self.volume_type['id'], extra_specs)['extra_specs']
        self.assertEqual(extra_specs, body)
        body = self.admin_volume_types_client.list_volume_types_extra_specs(
            self.volume_type['id'])['extra_specs']
        for key in extra_specs:
            self.assertIn(key, body)
            self.assertEqual(extra_specs[key], body[key],
                             "Volume type extra spec incorrectly created")

    @decorators.idempotent_id('d4772798-601f-408a-b2a5-29e8a59d1220')
    def test_volume_type_extra_spec_create_get_delete(self):
        """Test Create/Get/Delete volume type extra specs"""
        spec_key = "spec3"
        extra_specs = {spec_key: "val1"}
        body = self.admin_volume_types_client.create_volume_type_extra_specs(
            self.volume_type['id'],
            extra_specs)['extra_specs']
        self.assertEqual(extra_specs, body,
                         "Volume type extra spec incorrectly created")

        body = self.admin_volume_types_client.show_volume_type_extra_specs(
            self.volume_type['id'],
            spec_key)
        self.assertEqual(extra_specs, body,
                         "Volume type extra spec incorrectly fetched")

        self.admin_volume_types_client.delete_volume_type_extra_specs(
            self.volume_type['id'], spec_key)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_volume_types_client.show_volume_type_extra_specs,
            self.volume_type['id'], spec_key)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.compute import quotas_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestQuotasClient(base.BaseServiceTest):

    FAKE_QUOTA_SET = {
        "quota_set": {
            "injected_file_content_bytes": 10240,
            "metadata_items": 128,
            "server_group_members": 10,
            "server_groups": 10,
            "ram": 51200,
            "floating_ips": 10,
            "key_pairs": 100,
            "id": "8421f7be61064f50b680465c07f334af",
            "instances": 10,
            "security_group_rules": 20,
            "injected_files": 5,
            "cores": 20,
            "fixed_ips": -1,
            "injected_file_path_bytes": 255,
            "security_groups": 10}
        }

    project_id = "8421f7be61064f50b680465c07f334af"
    fake_user_id = "65f09168cbb04eb593f3138b63b67b67"

    def setUp(self):
        super(TestQuotasClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = quotas_client.QuotasClient(
            fake_auth, 'compute', 'regionOne')

    def _get_quota_set(self, detail):
        if not detail:
            return self.FAKE_QUOTA_SET
        fake_quota_set = {"quota_set": {}}
        for key, val in self.FAKE_QUOTA_SET['quota_set'].items():
            fake_quota_set['quota_set'][key] = \
                {'limit': val, 'reserved': 0, 'in_use': 0}
        fake_quota_set['quota_set']['id'] = "8421f7be61064f50b680465c07f334af"
        return fake_quota_set

    def _test_show_quota_set(self, bytes_body=False, detail=False,
                             user_id=None):
        if user_id:
            self.check_service_client_function(
                self.client.show_quota_set,
                'tempest.lib.common.rest_client.RestClient.get',
                self._get_quota_set(detail),
                to_utf=bytes_body,
                tenant_id=self.project_id,
                detail=detail,
                user_id=user_id)
        else:
            self.check_service_client_function(
                self.client.show_quota_set,
                'tempest.lib.common.rest_client.RestClient.get',
                self._get_quota_set(detail),
                to_utf=bytes_body,
                tenant_id=self.project_id,
                detail=detail)

    def test_show_quota_set_with_str_body(self):
        self._test_show_quota_set()

    def test_show_quota_set_with_bytes_body(self):
        self._test_show_quota_set(bytes_body=True)

    def test_show_quota_set_for_user_with_str_body(self):
        self._test_show_quota_set(user_id=self.fake_user_id)

    def test_show_quota_set_for_user_with_bytes_body(self):
        self._test_show_quota_set(bytes_body=True, user_id=self.fake_user_id)

    def test_show_quota_set_with_details(self):
        self._test_show_quota_set(detail=True)

    def _test_show_default_quota_set(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_default_quota_set,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_QUOTA_SET,
            to_utf=bytes_body,
            tenant_id=self.project_id)

    def test_show_default_quota_set_with_str_body(self):
        self._test_show_default_quota_set()

    def test_show_default_quota_set_with_bytes_body(self):
        self._test_show_default_quota_set(bytes_body=True)

    def _test_update_quota_set(self, bytes_body=False, user_id=None):
        fake_quota_set = copy.deepcopy(self.FAKE_QUOTA_SET)
        fake_quota_set['quota_set'].pop("id")
        if user_id:
            self.check_service_client_function(
                self.client.update_quota_set,
                'tempest.lib.common.rest_client.RestClient.put',
                fake_quota_set,
                to_utf=bytes_body,
                tenant_id=self.project_id,
                user_id=user_id)
        else:
            self.check_service_client_function(
                self.client.update_quota_set,
                'tempest.lib.common.rest_client.RestClient.put',
                fake_quota_set,
                to_utf=bytes_body,
                tenant_id=self.project_id)

    def test_update_quota_set_with_str_body(self):
        self._test_update_quota_set()

    def test_update_quota_set_with_bytes_body(self):
        self._test_update_quota_set(bytes_body=True)

    def test_update_quota_set_for_user_with_str_body(self):
        self._test_update_quota_set(user_id=self.fake_user_id)

    def test_update_quota_set_for_user_with_bytes_body(self):
        self._test_update_quota_set(bytes_body=True, user_id=self.fake_user_id)

    def test_delete_quota_set(self):
        self.check_service_client_function(
            self.client.delete_quota_set,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=202, tenant_id=self.project_id)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class EndpointsNegativeTestJSON(base.BaseIdentityV3AdminTest):
    """Negative tests of endpoint"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def setup_clients(cls):
        super(EndpointsNegativeTestJSON, cls).setup_clients()
        cls.client = cls.endpoints_client

    @classmethod
    def resource_setup(cls):
        prefix = CONF.resource_name_prefix
        super(EndpointsNegativeTestJSON, cls).resource_setup()
        s_name = data_utils.rand_name(name='service', prefix=prefix)
        s_type = data_utils.rand_name(name='type', prefix=prefix)
        s_description = data_utils.rand_name(name='description', prefix=prefix)
        service_data = (
            cls.services_client.create_service(name=s_name, type=s_type,
                                               description=s_description)
            ['service'])
        cls.addClassResourceCleanup(cls.services_client.delete_service,
                                    service_data['id'])

        cls.service_id = service_data['id']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ac6c137e-4d3d-448f-8c83-4f13d0942651')
    def test_create_with_enabled_False(self):
        """Test creating endpoint with invalid enabled value 'False'

        Enabled should be a boolean, not a string like 'False'
        """
        interface = 'public'
        url = data_utils.rand_url()
        region = data_utils.rand_name(
            name='region', prefix=CONF.resource_name_prefix)
        self.assertRaises(lib_exc.BadRequest, self.client.create_endpoint,
                          service_id=self.service_id, interface=interface,
                          url=url, region=region, enabled='False')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9c43181e-0627-484a-8c79-923e8a59598b')
    def test_create_with_enabled_True(self):
        """Test creating endpoint with invalid enabled value 'True'

        Enabled should be a boolean, not a string like 'True'
        """
        interface = 'public'
        url = data_utils.rand_url()
        region = data_utils.rand_name(
            name='region', prefix=CONF.resource_name_prefix)
        self.assertRaises(lib_exc.BadRequest, self.client.create_endpoint,
                          service_id=self.service_id, interface=interface,
                          url=url, region=region, enabled='True')

    def _assert_update_raises_bad_request(self, enabled):

        # Create an endpoint
        region1_name = data_utils.rand_name(
            name='region', prefix=CONF.resource_name_prefix)
        url1 = data_utils.rand_url()
        interface1 = 'public'
        endpoint_for_update = (
            self.client.create_endpoint(service_id=self.service_id,
                                        interface=interface1,
                                        url=url1, region=region1_name,
                                        enabled=True)['endpoint'])
        region1 = self.regions_client.show_region(region1_name)['region']
        self.addCleanup(self.regions_client.delete_region, region1['id'])
        self.addCleanup(self.client.delete_endpoint, endpoint_for_update['id'])

        self.assertRaises(lib_exc.BadRequest, self.client.update_endpoint,
                          endpoint_for_update['id'], enabled=enabled)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('65e41f32-5eb7-498f-a92a-a6ccacf7439a')
    def test_update_with_enabled_False(self):
        """Test updating endpoint with invalid enabled value 'False'

        Enabled should be a boolean, not a string like 'False'
        """
        self._assert_update_raises_bad_request('False')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('faba3587-f066-4757-a48e-b4a3f01803bb')
    def test_update_with_enabled_True(self):
        """Test updating endpoint with invalid enabled value 'True'

        Enabled should be a boolean, not a string like 'True'
        """
        self._assert_update_raises_bad_request('True')
